module WikiHelper
  include RailsConnector::DefaultCmsHelper

  def format_wiki_text(text)
    markdown(convert_wiki_links(text)).html_safe
  end

private

  def convert_wiki_links(text)
    text && text.gsub(/\[\[(.+?)\](\[(.+?)\])?\]/) do
      name, title = $1, $3
      generate_markdown_link(name, title)
    end
  end

  def generate_markdown_link(name, title)
    markdown_link = "[#{escape_markdown(title || name)}](#{generate_wiki_href(name)})"
    markdown_link << "{: .wiki_nonexisting_page}" unless @obj.wiki_pages.find_by_name(name)
    markdown_link
  end

  def generate_wiki_href(link)
    wiki_path(:id => @obj, :action => "show", :name => link)
  end

  def escape_markdown(markdown)
    markdown.gsub(/([-\\`\*_{}\[\]\(\)#\+\.!])/) do
      "\\#{$1}"
    end
  end

  def wiki_action_link_to(text, *args)
    link_to(content_tag(:span, text.html_safe), *args)
  end
end
