# This Model describes the behavior of all CMS objects having the class Voting.
# Votings 'have_many' <tt>Votes</tt>
class Voting < Obj
  has_many :votes, :dependent => :delete_all, :foreign_key => :obj_id

  # Returns an array of answers.
  def answers
    self.ip_votingAnswers.split(/\r?\n/)
  end


  def percent_for_answer(answer)
    return 0 if total_votes == 0
    count_for_answer(answer) * 100.0 / total_votes
  end

  def count_for_answer(answer)
    vote = votes.find_by_answer(answer)
    vote ? vote.count : 0
  end

  def voted?
    !votes.empty?
  end

  def reset_voting
    votes.clear
  end

  # Creates/adjusts votes for a particular Voting.
  def vote(answer)
    return nil if answer.blank?
    vote = votes.find_by_answer(answer) || votes.build(:answer => answer)
    vote.count += 1
    vote.save
    vote
  end

  def total_votes
    @total_votes ||= votes.collect(&:count).sum
  end

end