require "spec_helper"

describe VotesController, "AJAX /vote/2001 with choice 'Teddy'" do
  let(:voting) {mock_model(Obj)}
  let(:vote) {mock_model(Vote, :valid? => true)}

  before do
    Obj.stub(:find).and_return(voting)
    voting.stub(:vote).and_return(vote)
  end

  def do_ajax
    xhr :post, :vote, :obj_id => '2001', :choice => 'Teddy'
  end

  it "should fetch the voting object with ID 2001" do
    Obj.should_receive(:find).with("2001").and_return(voting)
    do_ajax
  end

  it "should vote for the given answer" do
    voting.should_receive(:vote).with('Teddy')
    do_ajax
  end

  it "should display the voting result" do
    do_ajax
    response.should render_template("_voting_area_content")
  end

  it "should remember that the user has voted in order to prevent the user from voting again" do
    lambda {
      do_ajax
    }.should change {
      controller.__send__(:current_user_has_already_voted?, voting)
    }.to(true)
  end
end

describe VotesController, "AJAX /vote/2001 without choice" do
  let(:voting) {mock_model(Obj, :vote => nil)}

  before do
    Obj.stub(:find).and_return(voting)
  end

  def do_ajax
    xhr :post, :vote, :obj_id => '2001', :choice => ''
  end

  it "should not vote" do
    do_ajax
    response.body.should be_blank
  end
end

describe VotesController, "GET /index when current user is admin" do
  before do
    controller.stub(:admin?).and_return(true)
  end

  def do_get
    get :index
  end

  it "should fetch all votings" do
    Voting.should_receive(:find).with(:all)
    do_get
  end
end


describe VotesController, "GET /index when current user is an ordinary user" do
  before do
    controller.stub(:admin?).and_return(false)
  end

  def do_get
    get :index
  end

  it "shouldn't fetch votings" do
    Obj.should_not_receive(:find_all_by_obj_class).with("Voting")
    do_get
  end

  it "should redirect to the homepage" do
    do_get
    response.should be_redirect
  end
end
