require "spec_helper"

require 'cms_helper'

describe CmsHelper, "homepage" do
  let(:homepage_obj) {mock_model(Obj, :mime_type => 'text/html', :id => 12345)}

  before(:each) do
    NamedLink.stub(:get_object).with("homepage_de").and_return(homepage_obj)
  end

  it "should return true if @obj is the homepage_de object" do
    helper.instance_variable_set("@obj", homepage_obj)
    helper.is_playland_homepage?.should be_true
  end

  it "should return false if @obj is nil" do
    helper.instance_variable_set("@obj",nil)
    helper.is_playland_homepage?.should be_false
  end

  it "should return false if @obj isn't the homepage object" do
    some_obj = mock_model(Obj, :mime_type => 'text/html', :id => 12345)
    helper.instance_variable_set("@obj",some_obj)
    helper.is_playland_homepage?.should be_false
  end
end

describe CmsHelper, "path_to_named_link" do
  let(:obj) {mock_model(Obj, :name => 'an_object', :id => 12345)}

  describe "when the named link exists" do
    before do
      NamedLink.stub(:get_object).with("some_named_link").and_return(obj)
      helper.stub(:cms_path)
    end

    let(:computed_path_to_named_link) {helper.path_to_named_link("some_named_link")}
    let(:html_safe_output) {computed_path_to_named_link}

    it_should_behave_like "an html safe helper"

    it "should return the object's cms path" do
      helper.should_receive(:cms_path).with(obj).and_return("/the/path")
      computed_path_to_named_link.should == "/the/path"
    end

  end

  it "shold raise an error if given named link does not exist" do
    NamedLink.stub(:get_object).with('does_not_exist').and_return nil
    lambda {
      helper.path_to_named_link("does_not_exist")
    }.should raise_error
  end

end

describe CmsHelper, "value_add_image" do
  let(:page) {mock_model(Obj, :name => 'page', :id => 1, :display_title => "d t")}

  before do
    helper.stub(:cms_path).and_return("/some/path".html_safe)
  end

  describe "when the page refers to an image" do
    let(:image) {obj = mock_model(Obj, :name => 'image', :id => 2)}
    let(:link_to_image) {mock_model(RailsConnector::Link, :destination_object => image)}

    before do
      stub_including_bracket_operator(page, :image_list => [link_to_image])
    end

    let(:rendered_with_image) {helper.value_add_image(page, :image_list)}
    let(:html_safe_output) {rendered_with_image}

    it_should_behave_like "an html safe helper"

  end
end

describe CmsHelper, "value_add_link_list" do
  let(:page) {mock_model(Obj, :name => 'page', :id => 2)}

  before do
    helper.stub(:cms_path)
    helper.stub(:link_list)
  end

  describe "when the page has value added links" do
    before do
      stub_including_bracket_operator(page, :related_links => [1, 2])
    end

    let(:html_safe_output) {
      helper.value_add_link_list(page, :related_links, "<html><for>header</for></html>")
    }

    it_should_behave_like "an html safe helper"
  end
end
