require "spec_helper"

require 'obj_extensions'

describe 'Object asked for its template' do

  before do
    NamedLink.stub(:get_object)
    @obj = Obj.new
  end

  describe 'if one of the reserved objects' do
    it "should return 'newsletter' if object is a Newsletter" do
      @obj.stub(:obj_class).and_return("Newsletter")
      @obj.template.should == "newsletter"
    end

    it "should return 'job_offer' if object is a JobOffer" do
      @obj.stub(:obj_class).and_return("JobOffer")
      @obj.template.should == "job_offer"
    end

    it "should return 'shop' if object is a vendor" do
      @obj.stub(:obj_class).and_return("Shop")
      @obj.template.should == "shop"
    end

    it "should return 'faq' if object is a Faq" do
      @obj.stub(:obj_class).and_return("Faq")
      @obj.template.should == "faq"
    end

    it "should return 'event' if object is a Event" do
      @obj.stub(:obj_class).and_return("Event")
      @obj.template.should == "event"
    end

  end

  describe 'if arbitrary object' do

    before do
      @obj.stub(:obj_class).and_return("something")
    end

    it "should return 'newsletter' if namedlink 'newsletter_de' points on obj" do
      NamedLink.stub(:get_object).with("newsletter_de").and_return(@obj)
      @obj.template.should == 'newsletter'
    end

    it "should return 'podcast' if namedlink 'podcast_de' points on obj" do
      NamedLink.stub(:get_object).with("podcast_de").and_return(@obj)
      @obj.template.should == 'podcasts'
    end

  end

  it "should return 'index' in all other cases" do
    @obj.stub(:obj_class).and_return("Publication")
    @obj.template.should == "index"
  end

end

describe "Obj", "seo_description" do
  it "should return a sanitized ip_description" do
    obj = Obj.new
    obj.stub(:[]).with(:ip_description).and_return("<abc>hi</abc> <p>foo</p> ")
    obj.seo_description.should == 'hi foo'
  end

  it "should return max 300 chars" do
    obj = Obj.new
    obj.stub(:[]).with(:ip_description).and_return("<p>123456789</p> " * 40)
    obj.seo_description.should == "123456789 " * 30
  end
end

describe "including OBJs in the SEO sitemap" do

  it "should not include objects that are above the homepage in the hierarchy" do
    Obj.homepage.ancestors.first.should_not be_included_in_seo_sitemap
    Obj.root.should_not be_included_in_seo_sitemap
  end

  it "should include the homepage" do
    Obj.homepage.should be_included_in_seo_sitemap
  end

  it "should include objects below the homepage" do
    Obj.homepage.toclist.first.should be_included_in_seo_sitemap
  end
end

describe "Obj.homepage" do
  it "should be the homepage object defined by named link" do
    Obj.homepage.should == NamedLink.get_object("homepage_de")
  end
end
