require "spec_helper"

describe Wiki do

  before do
    @obj = Wiki.new
    @obj.stub(:wiki_pages).and_return(@wiki_pages = mock("wiki pages association"))
    @page = mock_model(WikiPage)
  end

  it "should find the start page or create it if it doesn't exists" do
    @wiki_pages.should_receive(:find_or_create_by_name).with("StartPage").and_return(@page)
    @obj.find_or_create_start_page.should == @page
  end

  it "should find an existing wiki page based on a given name" do
    @wiki_pages.should_receive(:find_by_name).with("MyPage").and_return(@page)
    @obj.find_or_build_page("MyPage").should == @page
  end

  it "should build a nonexisting wiki page based on a given name" do
    @wiki_pages.should_receive(:find_by_name).with("MyPage").and_return(nil)
    @wiki_pages.should_receive(:build).with(:name => "MyPage").and_return(@page)
    @obj.find_or_build_page("MyPage").should == @page
  end

  it "should find all pages" do
    @wiki_pages.should_receive(:find).with(:all).and_return([@page])
    @obj.find_pages.should == [@page]
  end

  it "should delete a page with a given name" do
    @wiki_pages.should_receive(:find_by_name).with("MyPage").and_return(@page)
    @page.should_receive(:destroy)
    @obj.delete_page("MyPage")
  end

end

describe Wiki, "with real wiki pages" do

  before do
    @obj = Wiki.find(:first)
    WikiPage.delete_all
    WikiPage.create(:wiki_id => @obj.id, :name => 'x')
    WikiPage.create(:wiki_id => @obj.id, :name => 'a')
    WikiPage.create(:wiki_id => @obj.id, :name => 'm')
  end

  it "should have the wiki pages sorted by their name" do
    @obj.wiki_pages.collect { |page| page.name }.should == ["a", "m", "x"]
  end
end
