require "spec_helper"

describe "routing helper for wiki" do
  it "should have a named route 'wiki' supporting :id and :name" do
    wiki_path(:action => "show", :id => "123", :name => "page_name").
        should == "/wiki/123/show/page_name"
  end


  it "should support well_formed urls for umlaut page names" do
    wiki_path(:action => :show, :id => "123", :name => "www.my späce.de").
        should == "/wiki/123/show/www.my%20sp%C3%A4ce.de"
  end
end

describe "routing for wiki" do
  let(:path_to_show_named_page) {"/wiki/123/show/www.my%20sp%C3%A4ce.de"}

  it "considers name and action as optional" do
    {
      :get => "/wiki/123"
    }.should route_to({
      :controller => "wiki",
      :action => "index",
      :id => "123",
    })
  end

  describe "show page" do
    before do
      wiki_path(:action => :show, :id => "123", :name => "www.my späce.de").
          should == path_to_show_named_page
    end

    it "provides name and id from the show url" do
      {
        :get => path_to_show_named_page
      }.should route_to({
          :controller => "wiki",
          :action => "show",
          :id => "123",
          :name => "www.my späce.de",
      })
    end
  end
end
