require "spec_helper"

describe "/wiki/index" do

  let(:obj) {
    mock_model(Wiki, {
      :title => "Wiki Start Page",
      :to_param => "123",
    })
  }

  before do
    assign(:obj, obj)
    assign(:page, mock_model(WikiPage, :body => "BODY", :name => "StartPage"))
    view.stub(:display).and_return("foo")
    view.stub(:format_wiki_text).and_return("bar")
  end

  it "should display the editable Wiki description" do
    view.should_receive(:display).with(obj, :body).and_return("EDITABLE DESCRIPTION")
    render
    rendered.should have_stripped_text('div', "EDITABLE DESCRIPTION")
  end

  it "should display some formatted starter content" do
    view.should_receive(:format_wiki_text).with("BODY").and_return("FORMATTED BODY")
    render
    rendered.should have_stripped_text('div', "FORMATTED BODY")
  end

  it "should have a link to edit the page" do
    render
    rendered.should have_stripped_text("a[href='/wiki/123/edit/StartPage']", "Bearbeiten")
  end

  it "should have a link to list all pages" do
    render
    rendered.should have_stripped_text("a[href= '/wiki/123/list']", "Alle Wiki-Seiten")
  end
end
