require "spec_helper"

describe "/wiki/show" do
  shared_examples_for "show a page" do

    it "should have a link to edit the page" do
      render
      rendered.should have_stripped_text("a[href='/wiki/123/edit/Page']", "Bearbeiten")
    end

    it "should have a link to list all pages" do
      render
      rendered.should have_tag("a[href='/wiki/123/list']")
    end

    it "should have a link to the start page" do
      render
      rendered.should have_tag("a[href='/wiki/123']")
    end
  end

  let(:obj) {
    mock_model(Wiki, :to_param => "123")
  }

  let(:page) {
    mock_model(WikiPage, :body => "Some content for the other page", :name => "Page")
  }

  before do
    assign(:obj, obj)
    assign(:page, page)
    view.stub(:format_wiki_text).and_return("foo")
  end

  describe "for an existing page" do

    before do
      page.stub(:new_record?).and_return(false)
    end

    it_should_behave_like "show a page"

    it "should display the formatted Wiki page content" do
      view.should_receive(:format_wiki_text).with(page.body).and_return("content")
      render
      rendered.should have_stripped_text('div', "content")
    end

    it "should not render a blank slate" do
      render
      rendered.should_not have_tag("div[class='wiki_blank_slate']")
    end

    it "should have a confirming delete action" do
      render
      options = ""
      options << '[data-confirm]'
      options << '[data-method="delete"]'
      options << '[rel="nofollow"]'
      rendered.should have_stripped_text("a#{options}", "Löschen")
    end
  end


  describe "for a new page" do

    before do
      page.stub(:new_record?).and_return(true)
    end

    it_should_behave_like "show a page"

    it "should render a blank slate" do
      render
      rendered.should have_tag("div[class='wiki_blank_slate']")
    end

    it "should not have a delete link" do
      render
      rendered.should_not have_tag("a[data-remote]")
    end
  end
end
